=== modified file 'breezy/git/tests/test_dir.py'
--- breezy/git/tests/test_dir.py	2019-09-01 16:44:05 +0000
+++ breezy/git/tests/test_dir.py	2019-12-26 13:20:45 +0000
@@ -54,11 +54,17 @@
         gd = controldir.ControlDir.open('.')
         self.assertIsInstance(gd, dir.LocalGitDir)
 
+    def test_open_ref_parent(self):
+        r = GitRepo.init(".")
+        cid = r.do_commit(message=b"message", ref=b'refs/heads/foo/bar')
+        gd = controldir.ControlDir.open('.')
+        self.assertRaises(errors.NotBranchError, gd.open_branch, 'foo')
+
     def test_open_workingtree(self):
-        GitRepo.init(".")
+        r = GitRepo.init(".")
+        r.do_commit(message=b"message")
 
         gd = controldir.ControlDir.open('.')
-        raise TestSkipped
         wt = gd.open_workingtree()
         self.assertIsInstance(wt, workingtree.GitWorkingTree)
 

=== modified file 'breezy/git/transportgit.py'
--- breezy/git/transportgit.py	2019-11-02 23:19:18 +0000
+++ breezy/git/transportgit.py	2019-12-26 13:20:45 +0000
@@ -212,7 +212,11 @@
         except NoSuchFile:
             return None
         with f:
-            header = f.read(len(SYMREF))
+            try:
+                header = f.read(len(SYMREF))
+            except ReadError:
+                # probably a directory
+                return None
             if header == SYMREF:
                 # Read only the first line
                 return header + next(iter(f)).rstrip(b"\r\n")

=== modified file 'breezy/plugins/fastimport/exporter.py'
--- breezy/plugins/fastimport/exporter.py	2019-06-30 13:05:41 +0000
+++ breezy/plugins/fastimport/exporter.py	2019-12-26 13:20:45 +0000
@@ -512,7 +512,8 @@
                 yield commands.FileModifyCommand(
                     change.path[1].encode("utf-8"),
                     helpers.kind_to_mode('symlink', False),
-                    None, tree_new.get_symlink_target(change.path[1]))
+                    None, tree_new.get_symlink_target(
+                        change.path[1]).encode('utf-8'))
             elif change.kind[1] == 'directory':
                 if not self.plain_format:
                     yield commands.FileModifyCommand(

=== modified file 'breezy/plugins/fastimport/processors/generic_processor.py'
--- breezy/plugins/fastimport/processors/generic_processor.py	2018-11-18 19:16:07 +0000
+++ breezy/plugins/fastimport/processors/generic_processor.py	2019-12-01 16:21:41 +0000
@@ -202,12 +202,9 @@
 
     def _load_info_and_params(self):
         from .. import bzr_commit_handler
-        self._mode = bool(self.params.get('mode', 'default'))
-        self._experimental = self._mode == 'experimental'
-
         # This is currently hard-coded but might be configurable via
         # parameters one day if that's needed
-        repo_transport = self.repo.control_files._transport
+        repo_transport = self.repo.control_transport
         self.id_map_path = repo_transport.local_abspath("fastimport-id-map")
 
         # Load the info file, if any

=== modified file 'breezy/plugins/fastimport/tests/test_commands.py'
--- breezy/plugins/fastimport/tests/test_commands.py	2018-12-16 12:49:30 +0000
+++ breezy/plugins/fastimport/tests/test_commands.py	2019-12-01 16:14:53 +0000
@@ -23,6 +23,7 @@
 import gzip
 
 from .... import tests
+from ....tests import features
 from ....tests.blackbox import ExternalBase
 
 from ..cmds import (
@@ -155,6 +156,16 @@
         self.assertEquals("", data)
         self.assertPathExists("br.fi")
 
+    def test_symlink(self):
+        tree = self.make_branch_and_tree("br")
+        self.requireFeature(features.SymlinkFeature)
+        os.symlink('symlink-target', 'br/symlink')
+        tree.add('symlink')
+        tree.commit("add a symlink")
+        data = self.run_bzr("fast-export br br.fi")[0]
+        self.assertEquals("", data)
+        self.assertPathExists("br.fi")
+
     def test_tag_rewriting(self):
         tree = self.make_branch_and_tree("br")
         tree.commit("pointless")

=== modified file 'breezy/plugins/fastimport/tests/test_generic_processor.py'
--- breezy/plugins/fastimport/tests/test_generic_processor.py	2019-06-30 12:59:05 +0000
+++ breezy/plugins/fastimport/tests/test_generic_processor.py	2019-12-26 13:20:45 +0000
@@ -39,11 +39,8 @@
         ('pack-0.92', {'branch_format': 'pack-0.92'}),
         ('1.9-rich-root', {'branch_format': '1.9-rich-root'}),
     ]
-    try:
-        from ....bzr.groupcompress_repo import RepositoryFormat2a
-        scenarios.append(('2a', {'branch_format': '2a'}))
-    except ImportError:
-        pass
+    from ....bzr.groupcompress_repo import RepositoryFormat2a
+    scenarios.append(('2a', {'branch_format': '2a'}))
     suite = loader.suiteClass()
     result = tests.multiply_tests(standard_tests, scenarios, suite)
     return result

=== modified file 'doc/en/release-notes/brz-3.0.txt'
--- doc/en/release-notes/brz-3.0.txt	2019-12-23 01:39:21 +0000
+++ doc/en/release-notes/brz-3.0.txt	2019-12-26 13:20:45 +0000
@@ -32,6 +32,14 @@
 .. Fixes for situations where brz would previously crash or give incorrect
    or undesirable results.
 
+ * Don't throw an error attempting to read the parent of a ref.
+   (Jelmer Vernooĳ)
+
+ * Fix fast-exporting of symlinks on Python 3. (Jelmer Vernooĳ, #1851478)
+
+ * Don't rely on private transport attribute in fast-import.
+   (Jelmer Vernooĳ, #1854607)
+
 Documentation
 *************
 

